/* project-manager.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using GLib;

errordomain ProjectError
{
  MAX_OPEN
}

public class Valide.ProjectManager : Gtk.TreeView
{
  private static ProjectManager instance = null;
  private Project project;
  private GLib.List<Project> projects;
  private Gtk.TreeStore tree_store;
  private weak Project current_;
  private Gtk.Menu popup;

  private string copy_template (Template template, string path, string name)
                                                               throws GLib.Error
  {
    string filename = null;

    // Transform XML
    Xml.Doc* xml = Xml.Parser.parse_file (template.filename);
    Xml.XPathContext xpath = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath.eval ("/template");
    xpath_obj->nodesetval->item (0)->name = "project";

    xpath_obj = xpath.eval ("/project/description");
    xpath_obj->nodesetval->item (0)->unlink ();
    xpath_obj = xpath.eval ("/project/icon");
    xpath_obj->nodesetval->item (0)->unlink ();
    xpath_obj = xpath.eval ("/project/name");
    xpath_obj->nodesetval->item (0)->set_content (name);

    // Copy xml
    filename = GLib.Path.build_filename (path, name + Project.FILE_EXT);
    xml->save_file (filename);

    // Copy source files
    foreach (string file in template.files)
    {
      string contents = null;

      GLib.FileUtils.get_contents (file, out contents, null);
      GLib.FileUtils.set_contents (GLib.Path.build_filename (path, GLib.Path.get_basename (file)),
                                   contents, -1);
    }
    return filename;
  }

  public Project current
  {
    get
    {
      return this.current_;
    }
    set
    {
      this.current_ = value;
      this.refresh ();
    }
  }

  private ProjectManager ()
  {
    this.width_request = 200;
    this.headers_visible = false;
  }

  construct
  {
    this.tree_store = new Gtk.TreeStore (1, typeof (string));
    this.set_model (this.tree_store);
    Gtk.CellRendererText render = new Gtk.CellRendererText ();
    Gtk.TreeViewColumn column = new Gtk.TreeViewColumn.with_attributes (_("Projects"), render, "markup", 0, null);
    this.append_column (column);
    this.row_activated += this.file_select;
    /* Add popup */
    UIManager ui_manager = UIManager.get_instance ();
    ui_manager.add_ui_from_file (GLib.Path.build_filename (Config.PACKAGE_DATA_DIR, "valide-ui.xml"));
    this.popup = (Gtk.Menu)ui_manager.get_widget ("/project-popup");
    this.button_press_event += this.button_press;
    this.setup_ui ();
  }

  [Callback]
  private bool button_press (Gdk.EventButton event)
  {
    if (event.button == 3 && event.type == Gdk.EventType.BUTTON_PRESS)
    {
      this.popup.popup (null, null, null, event.button, event.time);
      return true;
    }
    return false;
  }

  [Callback]
  private void file_select (Gtk.TreeView sender, Gtk.TreePath path,
                              Gtk.TreeViewColumn column)
  {
    if (path.get_depth () == 2)
    {
      Gtk.TreeIter iter;
      string filename = null;

      this.tree_store.get_iter (out iter, path);
      this.tree_store.get (iter, 0, out filename);
      this.current.open_file (filename);
    }
    else
    {
      ///@todo Active project
    }
  }

  private void setup_ui ()
  {
    bool active = false;
    UIManager ui = UIManager.get_instance ();

    if (this.current != null)
    {
      active = true;
    }
    ui.get_action ("project-add").sensitive = active;
    ui.get_action ("project-remove").sensitive = active;
    ui.get_action ("project-options").sensitive = active;
    ui.get_action ("project-compile").sensitive = active;
    ui.get_action ("project-execute").sensitive = active;
    ui.get_action ("project-close").sensitive = active;
  }

  public void destroy ()
  {
    this.instance = null;
    base.destroy ();
  }

  public static ProjectManager get_instance ()
  {
    if (ProjectManager.instance == null)
    {
      ProjectManager.instance = new ProjectManager ();
    }
    return ProjectManager.instance;
  }

  public void create (string? filename = null) throws GLib.Error
  {
    string prj = null;

    if (current == null)
    {
      if (filename == null)
      {
        ProjectDialog dialog = new ProjectDialog ();

        if (dialog.run () == Gtk.ResponseType.OK)
        {
          bool create = false;
          string path = dialog.project_dir;

          if (!GLib.FileUtils.test (path, GLib.FileTest.EXISTS))
          {
            Gtk.Dialog msg = new Gtk.Dialog.with_buttons (_("New project"), null,
                                                          Gtk.DialogFlags.MODAL,
                                                          Gtk.STOCK_NO, Gtk.ResponseType.NO,
                                                          Gtk.STOCK_YES, Gtk.ResponseType.YES,
                                                          null);

            weak Gtk.Box main_box = (Gtk.Box)msg.vbox;
            main_box.pack_start (new Gtk.Label (_("Projects directory doesn't exist. Would you create it?")),
                                 true, true, 10);
            msg.show_all ();
            if (msg.run () == Gtk.ResponseType.YES)
            {
              GLib.DirUtils.create_with_parents (path, 0755);
              create = true;
            }
            msg.destroy ();
          }
          else
          {
            create = true;
          }

          if (create == true)
          {
            path = dialog.project_path;
            if (GLib.FileUtils.test (path, GLib.FileTest.EXISTS))
            {
              Gtk.Dialog msg = new Gtk.Dialog.with_buttons (_("New project"), null,
                                                            Gtk.DialogFlags.MODAL,
                                                            Gtk.STOCK_NO, Gtk.ResponseType.NO,
                                                            Gtk.STOCK_YES, Gtk.ResponseType.YES,
                                                            null);

              weak Gtk.Box main_box = (Gtk.Box)msg.vbox;
              main_box.pack_start (new Gtk.Label (_("Directory already exist. Would you overwrite it?")),
                                   true, true, 10);
              msg.show_all ();
              if (msg.run () == Gtk.ResponseType.NO)
              {
                create = false;
              }
              msg.destroy ();
            }
            else
            {
              GLib.DirUtils.create (path, 0755);
            }
          }

          if (create == true)
          {
            try
            {
              prj = this.copy_template (new Template (dialog.template),
                                             path, dialog.project_name);
            }
            catch (GLib.Error e)
            {
              GLib.warning (e.message);
            }
          }
        }
        dialog.destroy ();
      }
      else
      {
        prj = filename;
      }

      if (prj != null)
      {
        Project p = new Project (prj);

        RecentManagerUI.get_instance ().add_file (prj);
        p.closed += close;
        p.added_file += refresh;
        p.removed_file += refresh;
        this.projects.append (p);
        this.current = p;
        this.refresh ();
      }
    }
    else
    {
      throw new ProjectError.MAX_OPEN (_("Maximum project open!"));
    }
  }

  public void open ()
  {
    Gtk.FileChooserDialog dialog = new Gtk.FileChooserDialog (_("Open project"), null,
                                                              Gtk.FileChooserAction.OPEN,
                                                              Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                                                              Gtk.STOCK_OPEN, Gtk.ResponseType.ACCEPT,
                                                              null);

    /* Add filter */
    dialog.filter = new Gtk.FileFilter ();
    dialog.filter.set_name (_("Val(a)IDE project file"));
    dialog.filter.add_pattern ("*" + Project.FILE_EXT);

    if (dialog.run () == Gtk.ResponseType.ACCEPT)
    {
      this.create (dialog.get_filename ());
      Gtk.RecentManager recent_manager = Gtk.RecentManager.get_default ();
      recent_manager.add_item (dialog.get_filename ());
    }
    dialog.destroy ();
  }

  public void close (Project sender)
  {
    this.projects.remove (sender);
    this.current = null;
    this.refresh ();
  }

  private void refresh ()
  {
    this.tree_store.clear ();
    foreach (Project p in projects)
    {
      string title = null;
      Gtk.TreeIter root;

      this.tree_store.append (out root, null);

      if (p == this.current)
      {
        this.tree_store.set (root, 0, "<b>" + p.name + "</b>", -1);
      }
      else
      {
        this.tree_store.set (root, 0, p.name, -1);
      }
      foreach (string file in p.files)
      {
        Gtk.TreeIter iter;

        this.tree_store.append (out iter, root);
        this.tree_store.set (iter, 0, file, -1);
      }
    }
    this.setup_ui ();
  }
}

