/* executable-manager.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using GLib;

public class Valide.ExecutableManager : Gtk.HBox
{
  private static ExecutableManager instance = null;
  private Gtk.ListStore list_store;
  private Gtk.TreeView tree_view;

  public signal void cmd_start (string cmd);
  public signal void cmd_end (int status);
  public signal void parsing (ref string line);

  private string# text
  {
    get
    {
      string str = "";
      Gtk.TreeIter iter;

      if (this.list_store.get_iter_first (out iter) == true)
      {
        string val = null;

        do
        {
          this.list_store.get (iter, 0, &val, -1);
          str += val + "\n";
        } while (this.list_store.iter_next (ref iter));
      }
      return str;
    }
  }

  private ExecutableManager ()
  {
  }

  [Callback]
  private void clear (Gtk.Button sender)
  {
    this.list_store.clear ();
  }

  [Callback]
  private void copy (Gtk.Button sender)
  {
    weak Gtk.Clipboard clipboard = Gtk.Clipboard.get (Gdk.SELECTION_CLIPBOARD);
    string str = this.text;

    if (str != "")
    {
      clipboard.set_text (str, -1);
    }
    else
    {
      GLib.warning ("Nothing to copy!");
    }
  }

  [Callback]
  private void copy_bbcode (Gtk.Button sender)
  {
    weak Gtk.Clipboard clipboard = Gtk.Clipboard.get (Gdk.SELECTION_CLIPBOARD);
    string str = this.text;

    if (str != "")
    {
      GLib.HashTable<string, string> pattern = new GLib.HashTable<string, string> (str_hash, str_equal);
      pattern.insert ("<i>(.*)</i>", "[i]\1[/i]");
      pattern.insert ("<b>(.*)</b>", "[b]\1[/b]");
      pattern.insert ("<span background=\"(.*)\">(.*)</span>", "[color=\"\1\"]\2[/color]");
      pattern.insert ("<span foreground=\"(.*)\">(.*)</span>", "[color=\"\1\"]\2[/color]");

      foreach (string key in pattern.get_keys ())
      {
        GLib.Regex regex = new GLib.Regex (key,
                                           GLib.RegexCompileFlags.OPTIMIZE,
                                           GLib.RegexMatchFlags.NOTEMPTY);
        str = regex.replace (str, -1, 0, pattern.lookup (key), GLib.RegexMatchFlags.NOTEMPTY);

      }

      clipboard.set_text (str, -1);
    }
    else
    {
      GLib.warning ("Nothing to copy!");
    }
  }

  construct
  {
    Gtk.VBox vbox = new Gtk.VBox (false, 0);
    this.pack_start (vbox, false, false, 0);

    Gtk.Button button = new Gtk.Button ();
    button.relief = Gtk.ReliefStyle.NONE;
    button.image = new Gtk.Image.from_stock (Gtk.STOCK_CLEAR, Gtk.IconSize.SMALL_TOOLBAR);
    button.clicked += this.clear;
    vbox.pack_start (button, false, false, 0);

    button = new Gtk.Button ();
    button.relief = Gtk.ReliefStyle.NONE;
    button.image = new Gtk.Image.from_stock (Gtk.STOCK_COPY, Gtk.IconSize.SMALL_TOOLBAR);
    button.clicked += this.copy;
    vbox.pack_start (button, false, false, 0);

    button = new Gtk.Button ();
    button.relief = Gtk.ReliefStyle.NONE;
    button.image = new Gtk.Image.from_file (GLib.Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "bbcode.png"));
    button.clicked += this.copy_bbcode;
    vbox.pack_start (button, false, false, 0);
    button.set_sensitive (false);

    Gtk.ScrolledWindow scrolled_window = new Gtk.ScrolledWindow (null, null);
    this.list_store = new Gtk.ListStore (1, typeof (string));
    this.tree_view = new Gtk.TreeView.with_model (this.list_store);
    Gtk.CellRendererText render = new Gtk.CellRendererText ();
    Gtk.TreeViewColumn column = new Gtk.TreeViewColumn.with_attributes ("Message", render, "markup", 0, null);
    this.tree_view.append_column (column);
    this.tree_view.headers_visible = false;

    scrolled_window.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC);
    scrolled_window.add (this.tree_view);
    this.pack_start (scrolled_window, true, true, 0);
  }

  public static ExecutableManager get_instance ()
  {
    if (instance == null)
    {
      instance = new ExecutableManager ();
    }
    return instance;
  }

  private bool build_iofunc (IOChannel ioc, IOCondition cond)
  {
    bool ret = false;

    if ((cond & (GLib.IOCondition.IN | GLib.IOCondition.PRI)) != 0)
    {
      string msg;
      GLib.IOStatus rv;

      rv = ioc.read_line(out msg, null, null);
      if (msg != null)
      {
 	      this.add_line (msg);
 	    }
 	    if (rv == GLib.IOStatus.NORMAL || rv == GLib.IOStatus.AGAIN)
 	    {
  	    ret = true;
  	  }
    }
	  return ret;
  }

  private void cmd_watch (GLib.Pid pid, int status)
  {
    this.cmd_end (status);
  }

  public void run (string? dir, string cmd)
  {
    string cmd2;
    try
    {
      GLib.Regex regex = new GLib.Regex ("[ ]{2,}",
                                         GLib.RegexCompileFlags.OPTIMIZE,
                                         GLib.RegexMatchFlags.NOTEMPTY);
      cmd2 = regex.replace (cmd, -1, 0, " ", GLib.RegexMatchFlags.NOTEMPTY);
    }
    catch (GLib.Error e)
    {
      GLib.warning (e.message);
    }

    string[] args = cmd2.split (" ");
    int std_out, std_err;
    GLib.Pid pid;
    this.cmd_start (cmd2);
    GLib.Process.spawn_async_with_pipes (dir, args, null,
                                         GLib.SpawnFlags.SEARCH_PATH | GLib.SpawnFlags.DO_NOT_REAP_CHILD,
                                         null, out pid, null, out std_out,
                                         out std_err);
    GLib.IOCondition cond = GLib.IOCondition.IN | GLib.IOCondition.PRI
                          | GLib.IOCondition.ERR | GLib.IOCondition.HUP
                          | GLib.IOCondition.NVAL;
	  Utils.set_up_io_channel (std_out, cond, true, build_iofunc);
	  Utils.set_up_io_channel (std_err, cond, true, build_iofunc);
    GLib.ChildWatch.add (pid, this.cmd_watch);
  }

  public void add_line (string line)
  {
    GLib.Regex regex = new GLib.Regex ("[\n\r]*$",
                                       GLib.RegexCompileFlags.OPTIMIZE,
                                       GLib.RegexMatchFlags.NOTEMPTY);

    string line2 = regex.replace (line, -1, 0, "", GLib.RegexMatchFlags.NOTEMPTY);

    if (line2.len () > 0)
    {
      string[] lines = line2.split ("\n");
      
      foreach (string l in lines)
      {
        Gtk.TreeIter iter;

        this.parsing (ref l);

        this.list_store.append (out iter);
        this.list_store.set (iter, 0, l);

        Gtk.TreePath path = this.list_store.get_path (iter);
        this.tree_view.scroll_to_cell (path, null, false, 0, 0);
      }
    }
  }
}

