/* panel.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using GLib;

private class Valide.PanelItem 
{
  private const string KEY = "ValidePanelItemKey";
	public string name;
	public Gtk.Image icon;
}

public class Valide.Panel : Gtk.VBox
{
  private Gtk.Orientation orientation_;
  private Gtk.Notebook notebook;
  private Gtk.Image title_image;
  private Gtk.Label title_label;

  public signal void item_added (Gtk.Widget item);
  public signal void item_removed (Gtk.Widget item);
  public signal void close ();
  public signal void focus_document ();

  public Gtk.Orientation orientation
  {
    get
    {
      return this.orientation_;
    }
    set
    {
      this.orientation_ = value;
    	if (this.orientation == Gtk.Orientation.HORIZONTAL)
    	{
    		this.build_horizontal_panel ();
      }
    	else
    	{
    		this.build_vertical_panel ();
      }
    }
  }

  [Callback]
  private void panel_show ()
  {
    int page = this.notebook.get_current_page ();

    if (page != -1)
    {
//      this.notebook_page_changed (this.notebook, null, page);
    }
  }

  [Callback]
  private void notebook_page_changed (Gtk.Notebook sender,
                                      Gtk.NotebookPage? page,
                                      int page_num)
  {
    Gtk.Widget item = sender.get_nth_page (page_num);
    GLib.return_if_fail (item != null);

    weak PanelItem data = (PanelItem)item.get_data (PanelItem.KEY);
    GLib.return_if_fail (data != null);

    this.sync_title (data);
  }

  [Callback]
  private void button_style_set_cb (Gtk.Button sender)
  {
    int h, w;

    Gtk.icon_size_lookup_for_settings (sender.get_settings (), Gtk.IconSize.MENU,
                                       out w, out h);
    sender.set_size_request (w + 2, h + 2);
  }

  [Callback]
  private void close_button_clicked_cb ()
  {
	  this.hide ();
  }

  /* This is ugly, since it supports only known
   * storage types of GtkImage, otherwise fall back
   * to the empty icon.
   * See http://bugzilla.gnome.org/show_bug.cgi?id=317520.
   */
  private void set_gtk_image_from_gtk_image (Gtk.Image image, Gtk.Image source)
  {
    switch (source.get_storage_type ())
    {
      case Gtk.ImageType.EMPTY:
        image.clear ();
      break;
      case Gtk.ImageType.PIXMAP:
      {
        Gdk.Pixmap pm;
        Gdk.Bitmap bm;

        source.get_pixmap (out pm, out bm);
        image.set_from_pixmap (pm, bm);
      }
      break;
      case Gtk.ImageType.IMAGE:
      {
        Gdk.Image i;
        Gdk.Bitmap bm;

        source.get_image (out i, out bm);
        image.set_from_image (i, bm);
      }
      break;
      case Gtk.ImageType.PIXBUF:
      {
        Gdk.Pixbuf pb;

        pb = source.get_pixbuf ();
        image.set_from_pixbuf (pb);
      }
      break;
      case Gtk.ImageType.STOCK:
      {
        string s_id;
        Gtk.IconSize s;

        source.get_stock (out s_id, out s);
        image.set_from_stock (s_id, s);
      }
      break;
      case Gtk.ImageType.ICON_SET:
      {
        Gtk.IconSet i;
        Gtk.IconSize s;

        source.get_icon_set (out i, out s);
        image.set_from_icon_set (i, s);
      }
      break;
      case Gtk.ImageType.ANIMATION:
      {
        Gdk.PixbufAnimation a;

        a = source.get_animation ();
        image.set_from_animation (a);
      }
      break;
      case Gtk.ImageType.ICON_NAME:
      {
        string n;
        Gtk.IconSize s;

        source.get_icon_name (out n, out s);
        image.set_from_icon_name (n, s);
      }
      break;
      default:
        image.set_from_stock (Gtk.STOCK_FILE, Gtk.IconSize.MENU);
      break;
    }
  }

  private void sync_title (PanelItem? item)
  {
    if (this.orientation == Gtk.Orientation.HORIZONTAL
        && this.title_label != null)
    {
      if (item == null)
      {
        this.title_label.set_text (item.name);
        this.set_gtk_image_from_gtk_image (this.title_image, item.icon);
      }
      else
      {
        this.title_label.set_text ("Empty");
        this.title_image.set_from_stock (Gtk.STOCK_FILE, Gtk.IconSize.MENU);
      }
    }
  }

  private void build_notebook ()
  {
    /* Create the panel notebook */
    this.notebook = new Gtk.Notebook ();

    this.notebook.set_tab_pos (Gtk.PositionType.BOTTOM);
    this.notebook.scrollable = true;
    this.notebook.popup_enable ();
    this.notebook.show ();
    this.notebook.switch_page += notebook_page_changed;
  }

  private void build_horizontal_panel ()
  {
    /* Unused widget */
    this.title_label = new Gtk.Label ("Empty");
    this.title_image = new Gtk.Image ();

	  Gtk.HBox box = new Gtk.HBox (false, 0);
  	box.pack_start (this.notebook, true, true, 0);

  	/* Toolbar, close button and first separator */
	  Gtk.VBox sidebar = new Gtk.VBox (false, 6);
	  sidebar.set_border_width (4);

  	box.pack_start (sidebar, false, false, 0);

  	Gtk.Image image = new Gtk.Image.from_stock (Gtk.STOCK_CLOSE, Gtk.IconSize.MENU);
	  Gtk.Button close_button = this.create_small_button (image);
	  sidebar.pack_start (close_button, false, false, 0);
  	close_button.set_tooltip_text ("Hide panel");
	  close_button.clicked += close_button_clicked_cb;

    box.show_all ();

  	this.pack_start (box, true, true, 0);
  }

  private void build_vertical_panel ()
  {
  	/* Create title hbox */
	  Gtk.HBox title_hbox = new Gtk.HBox (false, 6);
	  title_hbox.set_border_width (5);
	  this.pack_start (title_hbox, false, false, 0);

  	Gtk.HBox icon_name_hbox = new Gtk.HBox (false, 0);
	  title_hbox.pack_start (icon_name_hbox, true, true, 0);

	  this.title_image = new Gtk.Image.from_stock (Gtk.STOCK_FILE, Gtk.IconSize.MENU);
	  icon_name_hbox.pack_start (this.title_image, false, true, 0);

  	Gtk.Label dummy_label = new Gtk.Label (" ");
	  icon_name_hbox.pack_start (dummy_label, false, false, 0);	

	  this.title_label = new Gtk.Label ("Empty");
	  this.title_label.set_alignment (0, (float)0.5);
	  this.title_label.set_ellipsize(Pango.EllipsizeMode.END);
	  icon_name_hbox.pack_start (this.title_label, true, true, 0);

	  Gtk.Image image = new Gtk.Image.from_stock (Gtk.STOCK_CLOSE, Gtk.IconSize.MENU);
	  Gtk.Button close_button = this.create_small_button (image);
	  close_button.set_tooltip_text ("Hide panel");
	  close_button.clicked += close_button_clicked_cb;
	  title_hbox.pack_start (close_button, false, false, 0);

	  title_hbox.show_all ();

	  this.pack_start (this.notebook, true, true, 0);
  }

  private Gtk.Button create_small_button (Gtk.Image image)
  {
    Gtk.Button button = new Gtk.Button ();
    button.set_relief (Gtk.ReliefStyle.NONE);

    /* don't allow focus on the close button */
    button.set_focus_on_click (false);

    /* make it as small as possible */
    Gtk.RcStyle rcstyle = new Gtk.RcStyle ();
    rcstyle.xthickness = rcstyle.ythickness = 0;
    button.modify_style (rcstyle);

    image.show ();

    button.add (image);

    /* Set minimal size */
    button.style_set += button_style_set_cb;

    return button;
  }

  private Gtk.Widget build_tab_label (Gtk.Widget item, string name,
                                      Gtk.Image icon)
  {
    /* set hbox spacing and label padding (see below) so that there's an
     * equal amount of space around the label */
    Gtk.HBox hbox = new Gtk.HBox (false, 4);

    Gtk.EventBox label_ebox = new Gtk.EventBox ();
    label_ebox.set_visible_window (false);
    hbox.pack_start (label_ebox, true, true, 0);

    Gtk.HBox label_hbox = new Gtk.HBox (false, 4);
    label_ebox.add (label_hbox);

    /* setup icon */
    label_hbox.pack_start (icon, false, false, 0);

    /* setup label */
    Gtk.Label label = new Gtk.Label (name);
    label.set_alignment ((float)0.0, (float)0.5);
    label.set_padding (0, 0);
    label_hbox.pack_start (label, true, true, 0);

    label_ebox.set_tooltip_text (name);

    hbox.show_all ();

    if (this.orientation == Gtk.Orientation.VERTICAL)
    {
      label.hide ();
    }
    item.set_data ("label", label);
    item.set_data ("hbox", hbox);
    item.set_data ("label-ebox", label_ebox);
    return hbox;
  }

  public Panel (Gtk.Orientation orientation)
  {
    this.orientation = orientation;
  }

  construct
  {
  	this.build_notebook ();
    this.show += panel_show;
  }

  public void add_item (Gtk.Widget item, string name, Gtk.Image? image=null)
  {
    PanelItem data = new PanelItem ();
    data.name = name;

    if (image == null)
    {
      /* default to empty */
      data.icon = new Gtk.Image.from_stock (Gtk.STOCK_FILE, Gtk.IconSize.MENU);
    }
    else
    {
      data.icon = image;
    }

    int w, h;
    Gtk.icon_size_lookup (Gtk.IconSize.MENU, out w, out h);
    data.icon.set_size_request (w, h);

    item.set_data (PanelItem.KEY, data);

    Gtk.Widget tab_label = this.build_tab_label (item, data.name, data.icon);

    Gtk.Label menu_label = new Gtk.Label (name);
    menu_label.set_alignment ((float)0.0, (float)0.5);

    if (!item.visible)
    {
      item.show ();
    }

    this.notebook.append_page_menu (item, tab_label, menu_label);
    this.item_added (item);
    this.title_label.set_text (name);
  }

  public void add_item_with_stock_icon (Gtk.Widget item, string name,
                                        string? stock_id)
  {
	  Gtk.Image icon = null;

	  if (stock_id != null)
	  {
		  icon = new Gtk.Image.from_stock (stock_id, Gtk.IconSize.MENU);
	  }
  	this.add_item (item, name, icon);
  }
}

