/* window.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using GLib;

public class Valide.Window : Gtk.Window
{
  private static Window instance = null;
  private UIManager ui_manager;
  private DocumentManager documents;
  private ProjectManager projects;
  private ExecutableManager executables;
  private Panel side_panel;
  private Panel bottom_panel;

  construct
  {
    this.maximize ();
    this.delete_event += on_delete_event;
    this.destroy += Gtk.main_quit;
    this.title = "Val(a)IDE";
    this.documents = DocumentManager.get_instance ();
    this.projects = ProjectManager.get_instance ();
    this.executables = ExecutableManager.get_instance ();
    this.setup_ui_manager ();
    this.populate_window ();

    try
    {
      GLib.List<Gdk.Pixbuf> list;
      list.append (new Gdk.Pixbuf.from_file (GLib.Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-16.xpm")));
      list.append (new Gdk.Pixbuf.from_file (GLib.Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-32.xpm")));
      list.append (new Gdk.Pixbuf.from_file (GLib.Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-48.xpm")));
      list.append (new Gdk.Pixbuf.from_file (GLib.Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "icone-64.xpm")));
      Gtk.Window.set_default_icon_list (list);
    }
    catch (GLib.Error e)
    {
      GLib.warning (e.message);
    }
  }

  [Callback]
  private bool on_delete_event ()
  {
    return !this.documents.close_all ();
  }

  private Window ()
  {
  }

	private void setup_ui_manager ()
	{
    this.ui_manager = UIManager.get_instance ();
    ui_manager.post_activate += this.on_action_activated;
    try
    {
      this.ui_manager.add_ui_from_file (GLib.Path.build_filename (Config.PACKAGE_DATA_DIR,
                                                                  "valide-ui.xml"));
    }
    catch (GLib.Error e)
    {
      GLib.warning (e.message);
    }
    this.add_accel_group (this.ui_manager.get_accel_group ());
  }

  private virtual void populate_window ()
  {
    Gtk.VBox main_box = new Gtk.VBox (false, 0);
    this.add (main_box);

		RecentManagerUI.get_instance ().update_file (this.ui_manager);
		RecentManagerUI.get_instance ().update_project (this.ui_manager);

    main_box.pack_start (this.ui_manager.get_widget ("/menubar"), false, true, 0);
    main_box.pack_start (this.ui_manager.get_widget ("/toolbar"), false, true, 0);

    Gtk.VPaned vpaned = new Gtk.VPaned ();
    main_box.pack_start (vpaned, true, true, 0);

    this.bottom_panel = new Panel (Gtk.Orientation.HORIZONTAL);
    this.bottom_panel.add_item_with_stock_icon (this.executables, "Executable",
                                           Gtk.STOCK_EXECUTE);
    vpaned.pack2 (this.bottom_panel, false, false);
    vpaned.set_position (50);

    Gtk.HPaned hpaned = new Gtk.HPaned ();
    vpaned.add1 (hpaned);

    this.side_panel = new Panel (Gtk.Orientation.VERTICAL);
    this.side_panel.add_item (this.projects, "Project");
    hpaned.add1 (this.side_panel);
    hpaned.add2 (this.documents);

    main_box.pack_start (new Gtk.Statusbar (), false, true, 0);
  }

  [Callback]
  private void on_action_activated (Gtk.UIManager sender, Gtk.Action action)
  {
    switch (action.name)
    {
      /* File menu */
      case "file-new":
        this.documents.create ();
      break;
      case "file-open":
        this.documents.open ();
      break;
      case "file-save":
        if (this.documents.current != null)
        {
          this.documents.current.save ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "file-save-as":
        if (this.documents.current != null)
        {
          this.documents.current.save_as ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "file-close":
        if (this.documents.current != null)
        {
          this.documents.current.close ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "file-close-all":
        if (this.documents.current != null)
        {
          this.documents.close_all ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "quit":
        if (!this.on_delete_event ())
          this.destroy ();
      break;

      /* Edit menu */
      case "edit-undo":
        if (this.documents.current != null)
        {
          this.documents.current.undo ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "edit-redo":
        if (this.documents.current != null)
        {
          this.documents.current.redo ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "edit-copy":
        if (this.documents.current != null)
        {
          this.documents.current.copy ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "edit-cut":
        if (this.documents.current != null)
        {
          this.documents.current.cut ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;
      case "edit-paste":
        if (this.documents.current != null)
        {
          this.documents.current.paste ();
        }
        else
        {
          GLib.warning (_("No document open!"));
        }
      break;

      /* View menu */
      case "view-side-panel":
        Gtk.ToggleAction toggle_action = (Gtk.ToggleAction)action;
        if (toggle_action.active)
        {
          this.side_panel.show ();
        }
        else
        {
          this.side_panel.hide ();
        }
      break;
      case "view-bottom-panel":
        Gtk.ToggleAction toggle_action = (Gtk.ToggleAction)action;
        if (toggle_action.active)
        {
          this.bottom_panel.show ();
        }
        else
        {
          this.bottom_panel.hide ();
        }
      break;

      /* Project menu */
      case "project-new":
        this.projects.create ();
      break;
      case "project-open":
        this.projects.open ();
      break;
      case "project-add":
        if (this.projects.current != null)
        {
          this.projects.current.add_file ();
        }
        else
        {
          GLib.warning (_("No project open!"));
        }
      break;
      case "project-remove":
        if (this.projects.current != null)
        {
          this.projects.current.remove_file ();
        }
        else
        {
          GLib.warning (_("No project open!"));
        }
      break;
      case "project-options":
        if (this.projects.current != null)
        {
          this.projects.current.options ();
        }
        else
        {
          GLib.warning (_("No project open!"));
        }
      break;
      case "project-compile":
        if (this.projects.current != null)
        {
          this.projects.current.compile ();
        }
        else
        {
          GLib.warning (_("No project open!"));
        }
      break;
      case "project-execute":
        if (this.projects.current != null)
        {
          this.projects.current.execute ();
        }
        else
        {
          GLib.warning (_("No project open!"));
        }
      break;
      case "project-close":
        if (this.projects.current != null)
        {
          this.projects.current.close ();
        }
        else
        {
          GLib.warning (_("No project open!"));
        }
      break;

      /* Help menu */
      case "help-about":
        var dialog = new AboutDialog ();
        dialog.run ();
        dialog.destroy ();
      break;
    }
  }

  public static Window get_instance ()
  {
    if (Window.instance == null)
    {
      Window.instance = new Window ();
    }
    return Window.instance;
  }

  public void run ()
  {
    this.show_all ();
    this.side_panel.hide ();
    this.bottom_panel.hide ();
  }
}

